@@ -1,5 +1,5 @@
 package PAR;
-$PAR::VERSION = '1.012';
+$PAR::VERSION = '1.013';
 
 use 5.006;
 use strict;
@@ -1088,7 +1088,7 @@ sub unpar {
 
     return $member if $member_only;
 
-    my ($fh, $LastTempFile) = _tempfile(
+    (my $fh, $LastTempFile) = _tempfile(
         sub { 
             my $fh = shift; 
             my $file = $member->fileName;
