@@ -32,6 +32,11 @@ environment variable.
 # for PAR internal use only!
 our $PARTemp;
 
+# name of the canary file
+our $Canary = "_CANARY_.txt";
+# how much to "date back" the canary file (in seconds)
+our $CanaryDateBack = 24 * 3600;        # 1 day
+
 # The C version of this code appears in myldr/mktmpdir.c
 # This code also lives in PAR::Packer's par.pl as _set_par_temp!
 sub set_par_temp_env {
@@ -98,7 +103,11 @@ sub _get_par_user_tempdir {
       qw( C:\\TEMP /tmp . )
   ) {
     next unless defined $path and -d $path and -w $path;
-    $temp_path = File::Spec->catdir($path, "par-$username");
+    # create a temp directory that is unique per user
+    # NOTE: $username may be in an unspecified charset/encoding;
+    # use a name that hopefully works for all of them;
+    # also avoid problems with platform-specific meta characters in the name
+    $temp_path = File::Spec->catdir($path, "par-".unpack("H*", $username));
     ($temp_path) = $temp_path =~ /^(.*)$/s;
     unless (mkdir($temp_path, 0700) || $!{EEXIST}) {
       warn "creation of private subdirectory $temp_path failed (errno=$!)"; 
@@ -114,7 +123,7 @@ sub _get_par_user_tempdir {
         if (!S_ISDIR($st[2])
             || $st[4] != $<
             || ($st[2] & 0777) != 0700 ) {
-          warn "private subdirectory $temp_path is unsafe";
+          warn "private subdirectory $temp_path is unsafe (please remove it and retry your operation)";
           return;
         }
     }
@@ -140,7 +149,6 @@ sub _find_username {
   else {
     $username = $ENV{USERNAME} || $ENV{USER} || 'SYSTEM';
   }
-  $username =~ s/\W/_/g;
 
   return $username;
 }
@@ -151,8 +159,6 @@ __END__
 
 =head1 SEE ALSO
 
-The PAR homepage at L<http://par.perl.org>.
-
 L<PAR>, L<PAR::Environment>
 
 =head1 AUTHORS
@@ -160,7 +166,7 @@ L<PAR>, L<PAR::Environment>
 Audrey Tang E<lt>cpan@audreyt.orgE<gt>,
 Steffen Mueller E<lt>smueller@cpan.orgE<gt>
 
-L<http://par.perl.org/> is the official PAR website.  You can write
+You can write
 to the mailing list at E<lt>par@perl.orgE<gt>, or send an empty mail to
 E<lt>par-subscribe@perl.orgE<gt> to participate in the discussion.
 
